import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:librus_api/librus_api.dart';
import 'package:librus_api/src/misc/activator.dart';
import 'package:librus_api/src/models/grade.dart';
import 'package:librus_api/src/models/resource.dart';
import 'dart:mirrors';

import 'package:librus_api/src/models/serializable.dart';

class Account with Serializable {
  int id;
  String group;
  String accessToken;
  String login;
  String studentName;
  String scopes;
  String state;
  String librusToken;
  List<Grade> grades;
//  Map<String, List<dynamic>> packages = {};

//  List<dynamic> pack<E extends Resource>(List<E> resources) {
//    return resources.map((Resource resource) {
//      dynamic data = {};
//      reflect(resource).type.declarations.values.forEach((value) {
//        String key = MirrorSystem.getName(value.simpleName);
//        if (value is VariableMirror) {
//          data[key] = reflect(resource).getField(value.simpleName).reflectee;
//        }
//      });
//      return data;
//    }).toList();
//  }
//
//  List<E> unpack<E extends Resource>(List<Map<String, dynamic>> input) {
//    return input.map((Map<String, dynamic> data) {
//      E resource = Activator.createInstance(E);
//      data.forEach((key, value) {
//        reflect(resource).setField(Symbol(key), value);
//      });
//      return resource;
//    }).toList();
//  }

  Future<List<E>> fetch<E extends Resource>() async {
    Resource resource = Activator.createInstance(E);
    String name = Activator.findStaticAndInvoke(resource, 'collection');
    String entry = Activator.findStaticAndInvoke(resource, 'entry');
    http.Response response =
        await http.get('${LibrusApi.apiUrl}/$name', headers: {
      'Authorization': 'Bearer $accessToken',
    });
    List<dynamic> itemsBody = json.decode(response.body)[entry];
    List<E> items = [];
    itemsBody.forEach((itemBody) {
      Resource item = Activator.createInstance(E);
      Map<String, dynamic> dynamicItem = itemBody;
      dynamicItem.forEach((key, value) {
        try {
          reflect(item).setField(Symbol(decapitalize(key)), value);
        } catch (err) {
          print("skipping setter ${decapitalize(key)}, because it not exists");
        }
      });
      items.add(item);
    });
    return items;
  }

  List<A> bind<A extends Resource, B extends Resource>(
      List<A> resources, List<B> targets, String key) {
    resources.forEach((Resource resource) {
      bool found = false;
      targets.forEach((Resource target) {
        if (!found &&
            reflect(resource).getField(Symbol(key)).reflectee['Id'] ==
                target.id) {
          reflect(resource).setField(Symbol(key), target);
          found = true;
        }
      });
    });
    return resources;
  }

  Future<String> refresh() async {
    http.Response response = await http.get(
        '${LibrusApi.baseUrl}/api/SynergiaAccounts/fresh/$login',
        headers: {
          'Authorization': 'Bearer $librusToken',
        });
    var data = json.decode(response.body);
    accessToken = data['accessToken'];
    return accessToken;
  }
}
