import 'dart:async';
import 'dart:mirrors';
import 'package:cookie_jar/cookie_jar.dart';
import 'package:html/parser.dart' show parse;
import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:http/http.dart' as http;
import 'package:librus_api/src/account.dart';
import 'package:librus_api/src/models/grade.dart';
import 'package:librus_api/src/models/serializable.dart';

class LibrusApi with Serializable {
  static final String baseUrl = 'https://portal.librus.pl';
  static final String clientId = 'wmSyUMo8llDAs4y9tJVYY92oyZ6h4lAt7KCuy0Gv';
  static final String apiUrl = 'https://api.librus.pl/2.0';
  String librusToken;

  Dio _client;

  LibrusApi(String tempDir) {
    _client = Dio(Options(headers: {'user-agent': 'LibrusMobileApp'}));
    _client.cookieJar = PersistCookieJar(tempDir);
  }

  Future<String> signIn(String email, String password) async {
    var csrfResponse = await this._client.get(
        '$baseUrl/oauth2/authorize?client_id=$clientId&redirect_uri=http://localhost/bar&response_type=code');
    var document = parse(csrfResponse.data);
    var csrfToken = document
        .querySelector('meta[name="csrf-token"][content]')
        .attributes['content'];
    await _client.post('$baseUrl/rodzina/login/action',
        data: json.encode({'email': email, 'password': password}),
        options: Options(headers: {
          'X-CSRF-TOKEN': csrfToken,
          'Content-Type': "application/json"
        }));
    var codeResponse = await _client.get(
        '$baseUrl/oauth2/authorize?client_id=$clientId&redirect_uri=http://localhost/bar&response_type=code',
        options: Options(
            followRedirects: false, validateStatus: (status) => status < 500));
    var authCode = codeResponse.headers.value('location').split('code=')[1];
    var exchangeToken = await _client.post(
      '$baseUrl/oauth2/access_token',
      data: {
        "grant_type": "authorization_code",
        "code": authCode,
        "client_id": LibrusApi.clientId,
        "redirect_uri": "http://localhost/bar"
      },
      options: Options(headers: {"Content-Type": "application/json"}),
    );
    librusToken = exchangeToken.data['access_token'];
    return librusToken;
  }

  Future<List<Account>> get accounts async {
    http.Response response =
        await http.get('$baseUrl/api/SynergiaAccounts', headers: {
      'Authorization': 'Bearer $librusToken',
    });
    List<dynamic> accountsBody = json.decode(response.body)['accounts'];
    List<Account> accounts = [];
    accountsBody.forEach((item) {
      Account account = new Account();
      account.id = item['id'];
      account.group = item['group'];
      account.accessToken = item['accessToken'];
      account.login = item['login'];
      account.studentName = item['studentName'];
      account.scopes = item['scopes'];
      account.state = item['state'];
      account.librusToken = librusToken;
      accounts.add(account);
    });
    requestedAccounts = accounts;
    return accounts;
  }

  LibrusApi.fromJson(Map<String, dynamic> json) {
    librusToken = json['librusToken'];
    requestedAccounts = json['requestedAccounts'];
  }

  List<Account> requestedAccounts;
}
