import 'dart:mirrors';

class Activator {
  static createInstance(Type type,
      [Symbol constructor,
      List arguments,
      Map<Symbol, dynamic> namedArguments]) {
    if (type == null) {
      throw new ArgumentError("type: $type");
    }

    if (constructor == null) {
      constructor = const Symbol("");
    }

    if (arguments == null) {
      arguments = const [];
    }

    var typeMirror = reflectType(type);
    if (typeMirror is ClassMirror) {
      return typeMirror
          .newInstance(constructor, arguments, namedArguments)
          .reflectee;
    } else {
      throw new ArgumentError(
          "Cannot create the instance of the type '$type'.");
    }
  }

  static dynamic findStaticAndInvoke(o, String name) {
    ClassMirror r = reflect(o).type;
    MethodMirror sFn;
    var s = new Symbol(name);

    while (sFn == null && r != null) {
      sFn = r.staticMembers[s];
      if (sFn != null) {
        break;
      }
      r = r.superclass;
    }

    if (sFn != null) {
      if (sFn.isGetter) {
        return r.getField(s).reflectee;
      } else {
        return r.invoke(s, []).reflectee;
      }
    }
  }
}

String decapitalize(String input) {
  if (input == null) {
    throw new ArgumentError("string: $input");
  }
  if (input.length == 0) {
    return input;
  }
  return input[0].toLowerCase() + input.substring(1);
}

String capitalize(String input) {
  if (input == null) {
    throw new ArgumentError("string: $input");
  }
  if (input.length == 0) {
    return input;
  }
  return input[0].toUpperCase() + input.substring(1);
}
